from distutils.core import setup
setup(name='PyFRS',# 包名
      version='0.0.9.1',
      description='Compute the multiband emissions from Forward-Reverse Shocks of a GRB or TDE jet',
      author='Wei-Hua Lei',
      author_email='leiwh78@gmail.com',
      packages=['PyFRS'],
      )
